// Copyright 2013 Google Inc. All Rights Reserved.
// You may study, modify, and use this example for any purpose.
// Note that this example is provided "as is", WITHOUT WARRANTY
// of any kind either expressed or implied.

var adsManager;
var adsLoader;
var adDisplayContainer;
var intervalTimer;
var playButton;
var videoContent;
var played = false;
var adplayed = false;
var playFixed = false;

var paused =  0;

function initAds(tag, xml) {
  //check if google is defined => otherwise adblock is detected
  if(typeof google === 'undefined') {
      if(!this.adblock){
          this.adblock = true;
          console.log('AdBlock detected');
          // ga('send', 'event', 'Adblock', 'blocked', 0);
      }
      return false; 
      
  }
        
  videoContent = document.getElementById('contentElement');
  setUpIMA(tag, xml);
  videoContent.onplay  = function() {
    //check if adsManager is loaded
    if(!playFixed){
      if(adsManager && !adplayed) playAds();
      playFixed = true;
    }

    //trigger play event once
    if(!played){
      console.log('play event');
      //shoot events
      // ga('send', 'event', 'Video', 'Play-NKP', vkey);
      // push_player_event("play");
      played = true;
    }

    //hide ad container
    // document.getElementById('adContainer').style.display = "none";
  };

  //ended event
  videoContent.ended  = function() {
    //trigger play event once
    console.log('Ended event');
    // ga('send', 'event', 'Video', 'ended', vkey);
    
  };  
}

function setUpIMA(tag, xml) {
  // Create the ad display container.
  createAdDisplayContainer();
  // Create ads loader.
  adsLoader = new google.ima.AdsLoader(adDisplayContainer);

  //disable ad breack autoplay
  adsLoader.getSettings().setAutoPlayAdBreaks(false);

  // Listen and respond to ads loaded and error events.
  adsLoader.addEventListener(
      google.ima.AdsManagerLoadedEvent.Type.ADS_MANAGER_LOADED,
      onAdsManagerLoaded,
      false);
  adsLoader.addEventListener(
      google.ima.AdErrorEvent.Type.AD_ERROR,
      onAdError,
      false);

  // An event listener to tell the SDK that our content video
  // is completed so the SDK can play any post-roll ads.
  var contentEndedListener = function() {adsLoader.contentComplete();};
  videoContent.onended = contentEndedListener;

  // Request video ads.
  var adsRequest = new google.ima.AdsRequest();
  
  if(tag) adsRequest.adTagUrl = tag;
  if(xml) adsRequest.adsResponse = xml;

  // Specify the linear and nonlinear slot sizes. This helps the SDK to
  // select the correct creative if multiple are returned.
  adsRequest.linearAdSlotWidth = window.innerWidth;
  adsRequest.linearAdSlotHeight = window.innerHeight;

  adsRequest.nonLinearAdSlotWidth = window.innerWidth;
  adsRequest.nonLinearAdSlotHeight = window.innerHeight;
  // adsRequest.forceNonLinearFullSlot = true;

  adsLoader.requestAds(adsRequest);
}


function createAdDisplayContainer() {
  //config lang
  google.ima.settings.setLocale('ar');
  // We assume the adContainer is the DOM id of the element that will house
  // the ads.
  adDisplayContainer = new google.ima.AdDisplayContainer(
  document.getElementById('adContainer'), videoContent);
}

function playAds() {
  // Initialize the container. Must be done via a user action on mobile devices.
  // videoContent.load();
  adDisplayContainer.initialize();

  try {
    // Initialize the ads manager. Ad rules playlist will start at this time.
    adsManager.init(width, height, google.ima.ViewMode.NORMAL);
    // Call play to start showing the ad. Single video and overlay ads will
    // start at this time; the call will be ignored for ad rules.
    adsManager.start();
  } catch (adError) {
    console.log('adError');
    console.log(adError);
    // An error may be thrown if there was a problem with the VAST response.
    if(!videoContent.ended) videoContent.play();
  }
}

function onAdsManagerLoaded(adsManagerLoadedEvent) {
  // Get the ads manager.
  var adsRenderingSettings = new google.ima.AdsRenderingSettings();
  adsRenderingSettings.restoreCustomPlaybackStateOnAdBreakComplete = true;
  
  //Render non-linear ads with a close and recall button 
  adsRenderingSettings.useStyledNonLinearAds = true;
  adsRenderingSettings.enablePreloading = true;

  // videoContent should be set to the content video element.
  adsManager = adsManagerLoadedEvent.getAdsManager(
      videoContent, adsRenderingSettings);

  // Add listeners to the required events.
  adsManager.addEventListener(
      google.ima.AdErrorEvent.Type.AD_ERROR,
      onAdError);
  adsManager.addEventListener(
      google.ima.AdEvent.Type.CONTENT_PAUSE_REQUESTED,
      onContentPauseRequested);
  adsManager.addEventListener(
      google.ima.AdEvent.Type.CONTENT_RESUME_REQUESTED,
      onContentResumeRequested);
  adsManager.addEventListener(
      google.ima.AdEvent.Type.ALL_ADS_COMPLETED,
      onAdEvent);

  adsManager.addEventListener(
      google.ima.AdEvent.Type.AD_BREAK_READY,
      onAdEvent);

  // Listen to any additional events, if necessary.
  adsManager.addEventListener(
      google.ima.AdEvent.Type.LOADED,
      onAdEvent);

  adsManager.addEventListener(
      google.ima.AdEvent.Type.LOG,
      onAdEvent);

  adsManager.addEventListener(
      google.ima.AdEvent.Type.STARTED,
      onAdEvent);

  adsManager.addEventListener(
      google.ima.AdEvent.Type.SKIPPED,
      onAdEvent);

  adsManager.addEventListener(
      google.ima.AdEvent.Type.IMPRESSION,
      onAdEvent);

  adsManager.addEventListener(
      google.ima.AdEvent.Type.COMPLETE,
      onAdEvent);

    // //fix mobile play action issue
  document.getElementById('mainContainer').addEventListener("click", function () {
    if(!playFixed){
      console.log('Click event')
      //shoot click ga event
      // ga('send', 'event', 'Video', 'ContainerClick-NKP', vkey);
      videoContent.play();
      if(adsManager && !adplayed) playAds();
      playFixed = true;
    }
  });
  
  //check if ad not yet played
  // if(!adplayed) playAds();
}

function onAdEvent(adEvent) {
  // Retrieve the ad from the event. Some events (e.g. ALL_ADS_COMPLETED)
  // don't have ad object associated.
  var ad = adEvent.getAd();
  console.log('ad', ad);

  switch (adEvent.type) {
    case google.ima.AdEvent.Type.LOADED:
      // This is the first event sent for an ad - it is possible to
      // determine whether the ad is a video ad or an overlay.
      if (!ad.isLinear()) {
        // Position AdDisplayContainer correctly for overlay.
        // Use ad.width and ad.height. => check if the video ended
        if(!videoContent.ended) videoContent.play();
        
        //lef up the ad container, so the user can manipulate the control bar
        document.getElementById('adContainer').style.top = "-80px";
      }
      else{
        //It's a linear ad => we put back the control bar top poition
        document.getElementById('adContainer').style.top = "0px";
      }
      break;

    case google.ima.AdEvent.Type.LOG:
      var adData = adEvent.getAdData();
      console.log(adData);
      if(adData.adError){
        var err = adData.adError;
        //ga tracking event
        // ga('send', 'event', 'Video', 'ADERROR-NKP-'+err.h, vkey);

        //check error type
        // if(err.h == 503) push_player_event("aderr_503");
        // else if(err.h == 1009) push_player_event("aderr_1009");
        // else if(err.h == 1012) push_player_event("aderr_1012");
        // else push_player_event("aderr_unknown");

        //hide ad container
        document.getElementById('adContainer').style.display = "none";
      }
    break;

    case google.ima.AdEvent.Type.IMPRESSION:
      // ga('send', 'event', 'Video', 'StartAds-NKP', vkey);
      // push_player_event('startAds');
    break;

    case google.ima.AdEvent.Type.AD_BREAK_READY:
      //init ad container display
      document.getElementById('adContainer').style.display = "block";

      adsManager.start();
    break;

    case google.ima.AdEvent.Type.ALL_ADS_COMPLETED:
      console.log("ALL_ADS_COMPLETED");
      //Hide ad container
      document.getElementById('adContainer').style.display = "none";
    break;

    case google.ima.AdEvent.Type.SKIPPED:
      console.log("Skipped");
      //Hide ad container
      document.getElementById('adContainer').style.display = "none";
    break;

    case google.ima.AdEvent.Type.STARTED:
      // This event indicates the ad has started - the video player
      // can adjust the UI, for example display a pause button and
      // remaining time.
      if (ad.isLinear()) {
        // For a linear ad, a timer can be started to poll for
        // the remaining time.
        intervalTimer = setInterval(
            function() {
              var remainingTime = adsManager.getRemainingTime();
            },
            300); // every 300ms
      }
    break;

    case google.ima.AdEvent.Type.COMPLETE:
      // This event indicates the ad has finished - the video player
      // can perform appropriate UI actions, such as removing the timer for
      // remaining time detection.
      if (ad.isLinear()) {
        clearInterval(intervalTimer);
      }
      
      //Hide ad container
      document.getElementById('adContainer').style.display = "none";
      
      break;
  }
}

function onAdError(adErrorEvent) {
  // Handle the error logging.
  var err = adErrorEvent.getError();
  console.log(err);
  //ga tracking event
  // ga('send', 'event', 'Video', 'ADERROR-NKP-'+err.h, vkey);

  //check error type
  // if(err.h == 503) push_player_event("aderr_503");
  // else if(err.h == 1009) push_player_event("aderr_1009");
  // else if(err.h == 1012) push_player_event("aderr_1012");
  // else push_player_event("aderr_unknown");

  if(adsManager) adsManager.destroy();
  //hide ad container
  document.getElementById('adContainer').style.display = "none";
}

function onContentPauseRequested() {
  videoContent.pause();
  // This function is where you should setup UI for showing ads (e.g.
  // display ad timer countdown, disable seeking etc.)
  // setupUIForAds();
  paused++;
}

function onContentResumeRequested() {
  if(!videoContent.ended) videoContent.play();
  // This function is where you should ensure that your UI is ready
  // to play content. It is the responsibility of the Publisher to
  // implement this function when necessary.
  // setupUIForContent();

  //hide ad container
  if(paused == 1) {paused++; document.getElementById('adContainer').style.display = "none";}

}

